(defun split-string (separator str)
  "Split a string at all occurrences of separator
   Returns a list."
  (let ((pos (position separator str)))
    (if pos
	(cons (subseq str 0 pos) 
	      (split-string separator (subseq str (1+ pos))))
	(list str))))

(defstruct account
  "Record to represent passwd lines"
  login-name
  password
  UID
  GID
  user-name
  directory
  shell)

(defun account-from-line (line)
  "Return an account record for an individual passwd line"
  (let ((fields (split-string #\: line)))
    (make-account
     :login-name (first fields) 
     :password (second fields)
     :UID (read-from-string (third fields))
     :GID (read-from-string (fourth fields))
     :user-name (fifth fields) 
     :directory (sixth fields) 
     :shell (seventh fields))))

(defun read-lines (f)
  "Read all lines of an open passwd file. 
   Return a list of account records"
  (let ((line (read-line f nil)))
    (if line
	(cons (account-from-line line) (read-lines f))
	())))
    
(defun read-passwd (filename)
  "Read the passwd file given in filename, return a list of account records"
  (with-open-file (f filename 
	 :external-format (ext:make-encoding 
			   :charset "iso-8859-1" 
			   :line-terminator :unix))
    (read-lines f)))

(defun find-account (passwd name)
  "Find an account with the login-name name in passwd."
  (find-if (lambda (a) (equal (account-login-name a) name)) passwd))